@@ -1,12 +1,3 @@
-/*
-Copyright ⓒ 2017 cargo-script contributors.
-
-Licensed under the MIT license (see LICENSE or <http://opensource.org
-/licenses/MIT>) or the Apache License, Version 2.0 (see LICENSE of
-<http://www.apache.org/licenses/LICENSE-2.0>), at your option. All
-files in the project carrying such notice may not be copied, modified,
-or distributed except according to those terms.
-*/
 /*!
 This module contains code related to template support.
 */
@@ -68,8 +59,8 @@ pub fn get_template_path() -> Result<PathBuf> {
         }
     }
 
-    let cache_path = platform::get_config_dir()?;
-    Ok(cache_path.join("script-templates"))
+    let cache_path = platform::templates_dir()?;
+    Ok(cache_path)
 }
 
 /**
